package com.yworks.eclipse.sample;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;
import y.base.Edge;
import y.view.EdgeRealizer;
import y.view.Graph2D;

/**
 * Selected edges are represented by this IPropertySource implementation.
 * This way a properties page can display the properties of a selected edge.
 */
public class EdgeProperty implements IPropertySource {
  private Graph2D graph;
  private Edge edge;
  private IPropertyDescriptor[] propertyDescriptors;

  public EdgeProperty( Edge edge ) {
    this.edge = edge;
    graph = ( Graph2D ) edge.getGraph();

    initPropertyDescriptors();
  }

  public void initPropertyDescriptors() {
    TextPropertyDescriptor label;
    propertyDescriptors = new IPropertyDescriptor[]{ label = new TextPropertyDescriptor( "LABEL_TEXT", "Label" ) };
    label.setCategory( "Outline" );
  }

  private Object getValue(String id ) {
    Object[] value = new Object[]{ null };

    try {
      EdgeRealizer r = graph.getRealizer( edge );
      if ( "LABEL_TEXT".equals( id ) ) {
        value[0] = r.getLabelText();
      }
    } catch ( Exception e ) {
      throw new RuntimeException( e );
    }

    return value[0];
  }

  private void setValue(final String id, final Object value) {
		EdgeRealizer r = graph.getRealizer(edge);
		if ("LABEL_TEXT".equals(id)) {
			r.setLabelText((String) value);
		}
		if ("FILL_COLOR1".equals(id)) {
			return;
		}
		if ("FILL_COLOR2".equals(id)) {
			return;
		}
		graph.updateViews();
	}

  public Object getEditableValue() {
    return null;
  }

  public IPropertyDescriptor[] getPropertyDescriptors() {
    return propertyDescriptors;
  }

  public Object getPropertyValue( Object id ) {
    return getValue( ( String ) id );
  }

  public boolean isPropertySet( Object id ) {
    if ( id.equals( "LABEL_TEXT" ) ) {
      return true;
    }
    return false;
  }

  public void resetPropertyValue( Object id ) {
  }

  public void setPropertyValue( Object id, Object value ) {
    setValue( ( String ) id, value );
  }

  public String toString() {
    return edge.toString();
  }


  public boolean equals( Object o ) {
    if ( o instanceof EdgeProperty && ( ( EdgeProperty ) o ).edge == edge ) {
      return true;
    }
    return false;
  }

  public int hashCode() {
    return edge.hashCode();
  }

  public Edge getEdge() {
    return edge;
  }
}
