package com.yworks.eclipse.sample;

import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;
import y.view.View;
import y.view.ViewChangeListener;
import y.view.ViewContainer;
import y.view.ViewControl;

import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

/**
 * The main plugin class. It manages resources, LnF, and default settings.
 */
public class GraphEditorPlugin extends AbstractUIPlugin implements ViewContainer {
  private static GraphEditorPlugin plugin;
  private ResourceBundle resourceBundle;

  public GraphEditorPlugin() {
    plugin = this;
    try {
      resourceBundle = ResourceBundle.getBundle( "com.yworks.eclipse.sample.GraphEditorResources" );
    } catch ( MissingResourceException ignore ) {
      resourceBundle = null;
    }

    setLookAndFeel();
  }

  public void setLookAndFeel() {
    SwingUtilities.invokeLater( new Runnable() {
      public void run() {
        try {
          if ( !"com.sun.java.swing.plaf.gtk.GTKLookAndFeel".equals( UIManager.getSystemLookAndFeelClassName() ) ) {
            UIManager.setLookAndFeel( UIManager.getSystemLookAndFeelClassName() );
          }
        } catch ( Exception e ) {
          e.printStackTrace();
        }
      }
    } );
  }

  /**
   * This method is called upon plug-in activation
   */
  public void start( BundleContext context ) throws Exception {
    super.start( context );
  }

  /**
   * This method is called when the plug-in is stopped
   */
  public void stop( BundleContext context ) throws Exception {
    super.stop( context );
  }

  /**
   * Returns the shared instance.
   */
  public static GraphEditorPlugin getDefault() {
    return plugin;
  }

  /**
   * Returns the string from the plugin's resource bundle,
   * or 'key' if not found.
   */
  public static String getResourceString( String key ) {
    ResourceBundle bundle = GraphEditorPlugin.getDefault().getResourceBundle();
    try {
      return ( bundle != null ) ? bundle.getString( key ) : key;
    } catch ( MissingResourceException e ) {
      return key;
    }
  }

  /**
   * Returns the plugin's resource bundle,
   */
  public ResourceBundle getResourceBundle() {
    return resourceBundle;
  }

  protected void initializeDefaultPluginPreferences() {
    getPreferenceStore().setDefault( "GENERAL_ANIMATE_ZOOMING", false );
    getPreferenceStore().setDefault( "GENERAL_ENABLE_ANTIALISING", true );
    getPreferenceStore().setDefault( "GENERAL_PAINT_NODES_OVER_EDGES", false );
    getPreferenceStore().setDefault( "GENERAL_DISPLAY_TOOLTIPS", false );
    getPreferenceStore().setDefault( "GENERAL_PAINT_DETAILS_THRESHOLD", 0.3 );
    getPreferenceStore().setDefault( "GENERAL_MAXIMUM_UNDO_DEPTH", 40 );

    getPreferenceStore().setDefault( "BACKGROUND_MODE", "PLAIN" );
    getPreferenceStore().setDefault( "BACKGROUND_COLOR", "" );

    getPreferenceStore().setDefault( "GRID_VISIBLE", false );
    getPreferenceStore().setDefault( "GRID_SNAP_TO_GRID", false );
    getPreferenceStore().setDefault( "GRID_SPACING", 24 );
    getPreferenceStore().setDefault( "GRID_TYPE", "Crosses" );
    getPreferenceStore().setDefault( "GRID_COLOR", "" );
  }

  public ViewControl getViewControl() {
    if ( getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor() instanceof GraphEditorPart ) {
      return ( ( GraphEditorPart ) getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor() ).getView().getViewControl();
    }
    return null;
  }

  public View getCurrentView() {
    if ( getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor() instanceof GraphEditorPart ) {
      return ( ( GraphEditorPart ) getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor() ).getView();
    }
    return null;
  }

  public void addViewChangeListener( ViewChangeListener arg0 ) {
  }

}
