package com.yworks.eclipse.sample;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.ColorFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.IntegerFieldEditor;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

/**
 * This preferences page allows to 
 * adjust some settings for <code>GraphEditorPart</code>.  
 **/
public class GraphEditorPreferences extends FieldEditorPreferencePage implements IWorkbenchPreferencePage {

  public GraphEditorPreferences() {
    super( GRID );
    setDescription( "yFiles Graph Editor Preferences" );
    setPreferenceStore( GraphEditorPlugin.getDefault().getPreferenceStore() );
  }

  protected void createFieldEditors() {
    GridLayout layout = new GridLayout();
    getFieldEditorParent().setLayout( layout );
    GridData data = new GridData( GridData.FILL_BOTH );
    CTabFolder folder = new CTabFolder( getFieldEditorParent(), SWT.BORDER );
    folder.setBorderVisible( true );
    folder.setLayoutData( data );

    folder.setTabHeight( 25 );
    folder.setSelectionBackground( new Color[]{ new Color( folder.getDisplay(), 249, 194, 97 ),
      new Color( folder.getDisplay(), 249, 249, 97 ) }, new int[]{ 100 } );

    // TODO: Not Windows Compatible
    // folder.setSimple(false);

    CTabItem generalItem = new CTabItem( folder, SWT.FLAT );
    folder.setSelection( 0 );
    generalItem.setText( "General" );

    Composite generalPage = new Composite( generalItem.getParent(), SWT.FLAT );
    layout = new GridLayout();
    layout.marginHeight = 20;
    layout.marginWidth = 10;
    layout.verticalSpacing = 10;
    generalPage.setLayout( layout );
    generalItem.setControl( generalPage );

    addField( new BooleanFieldEditor( "GENERAL_ANIMATE_ZOOMING", "Animate Zooming", SWT.NONE,
                                      generalPage ) );
    addField( new BooleanFieldEditor( "GENERAL_ENABLE_ANTIALISING", "Enable Antialising", SWT.NONE,
                                      generalPage ) );
    addField( new BooleanFieldEditor( "GENERAL_PAINT_NODES_OVER_EDGES", "Paint Nodes over Edges",
                                      SWT.NONE, generalPage ) );
    addField( new BooleanFieldEditor( "GENERAL_DISPLAY_TOOLTIPS", "Display ToolTips", SWT.NONE,
                                      generalPage ) );
    addField( new StringFieldEditor( "GENERAL_PAINT_DETAILS_THRESHOLD", "Paint Details Threshold",
                                     generalPage ) );
    addField( new IntegerFieldEditor( "GENERAL_MAXIMUM_UNDO_DEPTH", "Maximum Undo Steps", generalPage ) );

    CTabItem backgroundItem = new CTabItem( folder, SWT.FLAT );
    folder.setSelection( 0 );
    backgroundItem.setText( "Background" );

    Composite backgroundPage = new Composite( backgroundItem.getParent(), SWT.FLAT );
    layout = new GridLayout();
    layout.marginHeight = 10;
    layout.marginWidth = 10;
    backgroundPage.setLayout( layout );
    backgroundItem.setControl( backgroundPage );

    addField( new StringFieldEditor( "BACKGROUND_MODE", "Mode", backgroundPage ) );
    addField( new ColorFieldEditor( "BACKGROUND_COLOR", "Color", backgroundPage ) );

    CTabItem gridItem = new CTabItem( folder, SWT.FLAT );
    folder.setSelection( 0 );
    gridItem.setText( "Grid" );

    Composite gridPage = new Composite( gridItem.getParent(), SWT.FLAT );
    layout = new GridLayout();
    layout.marginHeight = 10;
    layout.marginWidth = 10;
    gridPage.setLayout( layout );
    gridItem.setControl( gridPage );

    addField( new BooleanFieldEditor( "GRID_VISIBLE", "Show Grid", SWT.NONE, gridPage ) );
    addField( new BooleanFieldEditor( "GRID_SNAP_TO_GRID", "Snap to Grid", SWT.NONE, gridPage ) );
    addField( new IntegerFieldEditor( "GRID_SPACING", "Spacing", gridPage ) );
    addField( new StringFieldEditor( "GRID_TYPE", "Type", gridPage ) );
    addField( new ColorFieldEditor( "GRID_COLOR", "Color", gridPage ) );

  }

  public void init( IWorkbench workbench ) {

  }

}