package com.yworks.eclipse.sample;

import java.util.ArrayList;
import java.util.List;

import javax.swing.SwingUtilities;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.widgets.Display;

import y.view.Graph2D;
import y.view.Graph2DView;
import y.view.Selections;

/**
 *
 *
 **/
public class GraphSelectionProvider implements ISelectionProvider {
  private List listeners = new ArrayList();
  private final Graph2DView view;
  private ISelection selection;

  public class MySelectionStateObserver extends Selections.SelectionStateObserver {
    protected void updateSelectionState(Graph2D graph) {
      GraphSelectionProvider.this.updateSelectionState();      
    }
  }
  
  public GraphSelectionProvider( Graph2DView view ) {
    this.view = view;
    MySelectionStateObserver observer = new MySelectionStateObserver();
    view.getGraph2D().addGraphListener(observer);
    view.getGraph2D().addGraph2DSelectionListener(observer);
    
    SwingUtilities.invokeLater( new Runnable() {
      public void run() {
        updateSelectionState();
      }
    } );
  }

  public Graph2DView getView() {
    return view;
  }

  public void addSelectionChangedListener( ISelectionChangedListener listener ) {
    listeners.add( listener );
  }

  public void removeSelectionChangedListener( ISelectionChangedListener listener ) {
    listeners.remove( listener );
  }

  public ISelection getSelection() {
    return selection;
  }

  public void setSelection( ISelection selection ) {
    this.selection = selection; //todo listeners???
  }

  /**
   * @EventDispatchThread
   */
  protected void updateSelectionState() {
    updateStructuredSelection();
    final SelectionChangedEvent sev = new SelectionChangedEvent( this, getSelection() );

    Display.getDefault().asyncExec( new Runnable() {
      public void run() {
        List copy = new ArrayList( listeners );
        for ( int i = 0; i < copy.size(); i++ ) {
          ( ( ISelectionChangedListener ) copy.get( i ) ).selectionChanged( sev );
        }
      }
    } );
  }

  /**
   * @EventDispatchThread
   */
  protected void updateStructuredSelection() {
    Graph2D graph = view.getGraph2D();
    this.selection = new GraphEditorPart.StructuredGraphSelection( graph );
  }
}
