package com.yworks.eclipse.sample;

import org.eclipse.swt.graphics.RGB;
import org.eclipse.ui.views.properties.ColorPropertyDescriptor;
import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;
import y.base.Node;
import y.base.NodeList;
import y.view.Graph2D;
import y.view.LineType;
import y.view.NodeRealizer;
import y.view.ShapeNodeRealizer;

import java.awt.Color;

/**
 * Selected nodes are represented by this IPropertySource implementation.
 * This way a properties page can display the properties of a selected node.
 */
public class NodeProperty implements IPropertySource {
  private Node node;
  private Graph2D graph;
  private IPropertyDescriptor[] propertyDescriptors;

  private static final String LINE_1 = "Line 1";
  private static final String LINE_2 = "Line 2";
  private static final String LINE_3 = "Line 3";
  private static final String LINE_4 = "Line 4";
  private static final String LINE_5 = "Line 5";

  private static final String DASHED_1 = "Dashed 1";
  private static final String DASHED_2 = "Dashed 2";
  private static final String DASHED_3 = "Dashed 3";
  private static final String DASHED_4 = "Dashed 4";
  private static final String DASHED_5 = "Dashed 5";

  private static final String DOTTED_1 = "Dotted 1";
  private static final String DOTTED_2 = "Dotted 2";
  private static final String DOTTED_3 = "Dotted 3";
  private static final String DOTTED_4 = "Dotted 4";
  private static final String DOTTED_5 = "Dotted 5";

  private static final String RECT = "Rectangle";
  private static final String ELLIPSE = "Ellipse";
  private static final String RECT3D = "Rectangle 3D";
  private static final String ROUND_RECT = "Round Rectangle";

  private static String[] lineTypes = new String[]{
    LINE_1, LINE_2, LINE_3, LINE_4, LINE_5,
    DASHED_1, DASHED_2, DASHED_3, DASHED_4, DASHED_5,
    DOTTED_1, DOTTED_2, DOTTED_3, DOTTED_4, DOTTED_5
  };

  private static String[] shapeTypes = new String[]{
    RECT, ELLIPSE, RECT3D, ROUND_RECT
  };

  public NodeProperty( Node node ) {
    this.node = node;
    graph = ( Graph2D ) node.getGraph();
    initPropertyDescriptors();
  }

  public LineType getLineType( Integer i2 ) {
    int i = i2.intValue();
    if ( i == 0 ) return LineType.LINE_1;
    if ( i == 1 ) return LineType.LINE_2;
    if ( i == 2 ) return LineType.LINE_3;
    if ( i == 3 ) return LineType.LINE_4;
    if ( i == 4 ) return LineType.LINE_5;

    if ( i == 5 ) return LineType.DASHED_1;
    if ( i == 6 ) return LineType.DASHED_2;
    if ( i == 7 ) return LineType.DASHED_3;
    if ( i == 8 ) return LineType.DASHED_4;
    if ( i == 9 ) return LineType.DASHED_5;

    if ( i == 10 ) return LineType.DOTTED_1;
    if ( i == 11 ) return LineType.DOTTED_2;
    if ( i == 12 ) return LineType.DOTTED_3;
    if ( i == 13 ) return LineType.DOTTED_4;
    if ( i == 14 ) return LineType.DOTTED_5;

    return LineType.LINE_1;
  }

  public Integer getShapeIndex( byte shape ) {
    if ( shape == ShapeNodeRealizer.RECT ) return new Integer( 0 );
    if ( shape == ShapeNodeRealizer.ELLIPSE ) return new Integer( 1 );
    if ( shape == ShapeNodeRealizer.RECT_3D ) return new Integer( 2 );
    if ( shape == ShapeNodeRealizer.ROUND_RECT ) return new Integer( 3 );
    return new Integer( 0 );
  }

  public byte getShapeType( Integer s ) {
    int shape = s.intValue();
    if ( shape == 0 ) return ShapeNodeRealizer.RECT;
    if ( shape == 1 ) return ShapeNodeRealizer.ELLIPSE;
    if ( shape == 2 ) return ShapeNodeRealizer.RECT_3D;
    if ( shape == 3 ) return ShapeNodeRealizer.ROUND_RECT;
    return ShapeNodeRealizer.RECT;
  }

  public Integer getLineTypeIndex( LineType t ) {
    if ( t == LineType.LINE_1 ) return new Integer( 0 );
    if ( t == LineType.LINE_2 ) return new Integer( 1 );
    if ( t == LineType.LINE_3 ) return new Integer( 2 );
    if ( t == LineType.LINE_4 ) return new Integer( 3 );
    if ( t == LineType.LINE_5 ) return new Integer( 4 );

    if ( t == LineType.DASHED_1 ) return new Integer( 5 );
    if ( t == LineType.DASHED_2 ) return new Integer( 6 );
    if ( t == LineType.DASHED_3 ) return new Integer( 7 );
    if ( t == LineType.DASHED_4 ) return new Integer( 8 );
    if ( t == LineType.DASHED_5 ) return new Integer( 9 );

    if ( t == LineType.DOTTED_1 ) return new Integer( 10 );
    if ( t == LineType.DOTTED_2 ) return new Integer( 11 );
    if ( t == LineType.DOTTED_3 ) return new Integer( 12 );
    if ( t == LineType.DOTTED_4 ) return new Integer( 13 );
    if ( t == LineType.DOTTED_5 ) return new Integer( 14 );

    return new Integer( 0 );
  }


  public void initPropertyDescriptors() {
    NodeList nL = new NodeList();
    nL.add( node );

    TextPropertyDescriptor label;
    TextPropertyDescriptor x;
    TextPropertyDescriptor y;
    TextPropertyDescriptor width;
    TextPropertyDescriptor height;
    ColorPropertyDescriptor fillColor1;
    ColorPropertyDescriptor fillColor2;
    ColorPropertyDescriptor textColor;

    ComboBoxPropertyDescriptor lineType;
    ComboBoxPropertyDescriptor shapeType;

    propertyDescriptors = new IPropertyDescriptor[]{
      label = new TextPropertyDescriptor( "LABEL_TEXT", "Label" ),
      shapeType = new ComboBoxPropertyDescriptor( "SHAPE_TYPE", "Shape Type", shapeTypes ),
      lineType = new ComboBoxPropertyDescriptor( "LINE_TYPE", "Line Type", lineTypes ),
      x = new TextPropertyDescriptor( "POSITION_X", "x" ),
      y = new TextPropertyDescriptor( "POSITION_Y", "y" ),
      width = new TextPropertyDescriptor( "NODE_WIDTH", "width" ),
      height = new TextPropertyDescriptor( "NODE_HEIGHT", "height" ),
      fillColor1 = new ColorPropertyDescriptor( "FILL_COLOR1", "Fill Color" ),
      fillColor2 = new ColorPropertyDescriptor( "FILL_COLOR2", "Fill Color 2" ),
      textColor = new ColorPropertyDescriptor( "TEXT_COLOR", "Text Color" ),
    };

    // Outline Category
    shapeType.setCategory( "Outline" );
    lineType.setCategory( "Outline" );
    fillColor1.setCategory( "Outline" );
    fillColor2.setCategory( "Outline" );

    // Position Category
    x.setCategory( "Position" );
    y.setCategory( "Position" );
    width.setCategory( "Position" );
    height.setCategory( "Position" );

    // Label Category
    label.setCategory( "Label" );
    textColor.setCategory( "Label" );

  }


  private Object getValue(final String id) {
	  final Object[] value = new Object[] { null };

		NodeRealizer r = graph.getRealizer(node);
		if ("LABEL_TEXT".equals(id)) {
			value[0] = r.getLabelText();
			return value[0];
		}
		if ("SHAPE_TYPE".equals(id)) {
			if (r instanceof ShapeNodeRealizer) {
				value[0] = getShapeIndex(((ShapeNodeRealizer) r).getShapeType());
				return value[0];
			} else {
				value[0] = getShapeIndex(ShapeNodeRealizer.RECT);
				return value[0];
			}
		}
		if ("FILL_COLOR1".equals(id)) {
			value[0] = new RGB(r.getFillColor().getRed(), r.getFillColor()
					.getGreen(), r.getFillColor().getBlue());
			return value[0];
		}
		if ("FILL_COLOR2".equals(id)) {
			if (r.getFillColor2() == null)
				return value[0];
			value[0] = new RGB(r.getFillColor2().getRed(), r.getFillColor2()
					.getGreen(), r.getFillColor2().getBlue());
			return value[0];
		}
		if ("TEXT_COLOR".equals(id)) {
			value[0] = new RGB(r.getLabel().getTextColor().getRed(), r
					.getLabel().getTextColor().getGreen(), r.getLabel()
					.getTextColor().getBlue());
			return value[0];
		}
		if ("LINE_TYPE".equals(id)) {
			value[0] = getLineTypeIndex(r.getLineType());
			return value[0];
		}
		if ("POSITION_X".equals(id)) {
			value[0] = String.valueOf(r.getCenterX());
			return value[0];
		}
		if ("POSITION_Y".equals(id)) {
			value[0] = String.valueOf(r.getCenterY());
			return value[0];
		}
		if ("NODE_WIDTH".equals(id)) {
			value[0] = String.valueOf(r.getWidth());
			return value[0];
		}
		if ("NODE_HEIGHT".equals(id)) {
			value[0] = String.valueOf(r.getHeight());
			return value[0];
		}
		return value[0];
	}

  private void setValue(final String id, final Object value) {
		NodeRealizer r = graph.getRealizer(node);
		if ("LABEL_TEXT".equals(id)) {
			r.setLabelText((String) value);
		} else if ("SHAPE_TYPE".equals(id)) {
			if (r instanceof ShapeNodeRealizer) {
				((ShapeNodeRealizer) r).setShapeType(getShapeType((Integer) value));
			}
		} else if ("FILL_COLOR1".equals(id)) {
			RGB rgb = (RGB) value;
			r.setFillColor(new Color(rgb.red, rgb.green, rgb.blue));
		} else if ("FILL_COLOR2".equals(id)) {
			RGB rgb = (RGB) value;
			r.setFillColor2(new Color(rgb.red, rgb.green, rgb.blue));
		} else if ("TEXT_COLOR".equals(id)) {
			RGB rgb = (RGB) value;
			r.getLabel().setTextColor(new Color(rgb.red, rgb.green, rgb.blue));
		} else if ("POSITION_X".equals(id)) {
			r.setCenterX(Double.valueOf((String) value).doubleValue());
		} else if ("POSITION_Y".equals(id)) {
			r.setCenterY(Double.valueOf((String) value).doubleValue());
		} else if ("NODE_WIDTH".equals(id)) {
			r.setWidth(Double.valueOf((String) value).doubleValue());
		} else if ("NODE_HEIGHT".equals(id)) {
			r.setHeight(Double.valueOf((String) value).doubleValue());
		} else if ("LINE_TYPE".equals(id)) {
			r.setLineType(getLineType((Integer) value));
		}
		graph.updateViews();
	}

  public Object getEditableValue() {
    return null;
  }

  public IPropertyDescriptor[] getPropertyDescriptors() {
    return propertyDescriptors;
  }

  public Object getPropertyValue( Object id ) {
    return getValue( ( String ) id );
  }

  public boolean isPropertySet( final Object id ) {
    final boolean[] value = new boolean[]{ false };
    try {
			if ("LABEL_TEXT".equals(id)) {
				value[0] = true;
			}
			if ("FILL_COLOR2".equals(id)) {
				value[0] = graph.getRealizer(node).getFillColor2() != null;
			}
    } catch ( Exception e ) {
      throw new RuntimeException( e );
    }
    return value[ 0 ];
  }

  public void resetPropertyValue(final Object id) {
		if ("FILL_COLOR2".equals(id)) {
			graph.getRealizer(node).setFillColor2(null);
		}
	}

  public void setPropertyValue( Object id, Object value ) {
    setValue( ( String ) id, value );
  }

  public Graph2D getGraph() {
    return graph;
  }


  public Node getNode() {
    return node;
  }

  public String toString() {
    return node.toString();
  }

  public boolean equals( Object o ) {
    if ( o instanceof NodeProperty && ( ( NodeProperty ) o ).node == node ) {
      return true;
    }
    return false;
  }

  public int hashCode() {
    return node.hashCode();
  }
}
