package com.yworks.eclipse.sample.actions;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import org.eclipse.albireo.core.AwtEnvironment;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorPart;

import y.module.YModule;
import y.option.OptionHandler;

import com.yworks.eclipse.sample.GraphEditorPart;
import com.yworks.eclipse.sample.GraphEditorPlugin;

/**
 * Action that launches a yFiles module. The OptionHandler instance 
 * associated of the YModule will be displayed inside a SWT dialog using
 * Albireo.
 */
public abstract class LaunchModuleAction implements IEditorActionDelegate {
  protected GraphEditorPart editor;
  YModule module;

  protected LaunchModuleAction() {}

  public void run( IAction action ) {
    if ( editor == null ) return;

    Display display = GraphEditorPlugin.getDefault().getWorkbench().getDisplay();
    display.asyncExec( new Runnable() {
      public void run() {
        OptionHandlerDialog ohd = new OptionHandlerDialog( editor.getSite().getShell() );
        String title = "untitled yFiles module";
        if(getModule() != null) {
        	title = getModule().getModuleName();
        }
        ohd.open(title);
      }
    } );
  }

  public YModule getModule() {
    if ( module == null ) module = createModule();
    return module;
  }

  protected abstract YModule createModule();

  public void selectionChanged( IAction action, ISelection selection ) {}

  public void setActiveEditor( IAction action, IEditorPart targetEditor ) {
    if ( targetEditor instanceof GraphEditorPart ) {
      editor = ( GraphEditorPart ) targetEditor;
    } else {
      editor = null;
    }
  }
  
  private void runModule() {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				YModule m = getModule();
				if (m != null) {
					m.start(editor.getView().getGraph2D());
				}
			}
		});
	}

  class OptionHandlerDialog extends Dialog implements ActionListener {
    private JDialog dialog;

    OptionHandlerDialog( Shell parent, int style ) {
      super( parent, style );
    }

    OptionHandlerDialog( Shell parent ) {
      this( parent, 0 );
    }

    public void open(final String title) {
			final Shell parent = getParent();
			OptionHandler oh = getModule().getOptionHandler();
			JComponent editor = oh.getEditor();
			final JPanel panel = new JPanel(new BorderLayout());
			panel.add(editor, BorderLayout.CENTER);

			JPanel buttons = new JPanel(new FlowLayout(FlowLayout.RIGHT, 12, 12));
			JButton button;
			button = new JButton("Ok");
			button.addActionListener(this);
			buttons.add(button);
			button = new JButton("Reset");
			button.addActionListener(this);
			buttons.add(button);
			button = new JButton("Cancel");
			button.addActionListener(this);
			buttons.add(button);
			panel.add(buttons, BorderLayout.SOUTH);

			AwtEnvironment env = AwtEnvironment.getInstance(parent.getDisplay());
			final Frame frame = env.createDialogParentFrame(parent.getShell());
			Runnable runnable = new Runnable() {
				public void run() {
					dialog = new JDialog(frame, title, true);
					dialog.add(panel);
					dialog.pack();
					dialog.setVisible(true);
				}
			};
			env.invokeAndBlockSwt(runnable);
		}

    public void actionPerformed( ActionEvent e ) {
      OptionHandler oh = getModule().getOptionHandler();
      if ( e.getActionCommand().equals( "Cancel" ) ) {
        closeDialog();
      } else if ( e.getActionCommand().equals( "Ok" ) ) {
        if ( oh.checkValues() ) {
          oh.commitValues();
          runModule();
          closeDialog();
        }
      } else if ( e.getActionCommand().equals( "Reset" ) ) {
        oh.resetValues();
      }
    }

    void closeDialog() {
    	dialog.dispose();
    }
  }
}